/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Calendar;
import java.util.Date;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.DayEndPrice;
import org.ferris.yahoo.finance.jmce.bean.Market;

/**
 *
 * @author Michael Remijan [mjremijan@yahoo.com]
 */
public class DayEndPriceHandlerBySql extends AbstractHandlerBySql implements DayEndPriceHandler
{
	private static final Logger log = Logger.getLogger(DayEndPriceHandlerBySql.class);
	
	public CalendarHandler calendarHandler;
	public CalendarHandler getCalendarHandler() {
		return calendarHandler;
	}
	public void setCalendarHandler(CalendarHandler calendarHandler) {
		this.calendarHandler = calendarHandler;
	}
    
    private PrimaryKeyHandler primaryKeyHandler;    
    public PrimaryKeyHandler getPrimaryKeyHandler() {
        return primaryKeyHandler;
    }
    public void setPrimaryKeyHandler(PrimaryKeyHandler primaryKeyHandler) {
        this.primaryKeyHandler = primaryKeyHandler;
    } 
	
	@Override
	public DayEndPrice getDayEndPriceForGold(Calendar day) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getDayEndPriceForGold(%s)",day));
		}		
		
		return getDayEndPriceForMarket(day, Market.Code.gc);
	}
	
	@Override
	public DayEndPrice getDayEndPriceForOil(Calendar day) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getDayEndPriceForOil(%s)",day));
		}		
		
		return getDayEndPriceForMarket(day, Market.Code.cl);
	}
	
	@Override
	public DayEndPrice getDayEndPriceForStandardAndPoors500(Calendar day) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getDayEndPriceForStandardAndPoors500(%s)",day));
		}		
		
		return getDayEndPriceForMarket(day, Market.Code.sp);
	}
	
	@Override
	public DayEndPrice getDayEndPriceForNasdaq(Calendar day) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getDayEndPriceForNasdaq(%s)",day));
		}		
		
		return getDayEndPriceForMarket(day, Market.Code.nas);
	}
	
	
	@Override
	public DayEndPrice getDayEndPriceForDow(Calendar day) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getDayEndPriceForDow(%s)",day));
		}
		
		return getDayEndPriceForMarket(day, Market.Code.dow);
	}
	
		
	private DayEndPrice getDayEndPriceForMarket(Calendar day, Market.Code marketCode) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getDayEndPriceForMarket(%s,%s)",day,marketCode));
		}
		        
		StringBuilder sp = new StringBuilder();
		sp.append(String.format(" select "));
		sp.append(String.format("		dep.id, dep.created_on, dep.updated_on, m.id as market_id, dep.day, dep.price "));
		sp.append(String.format(" from "));
		sp.append(String.format("   day_end_prices dep "));
        sp.append(String.format(" , markets m "));
		sp.append(String.format(" where "));                
        sp.append(String.format("		dep.market_ptr=m.id "));        		        
		sp.append(String.format("		and "));
        sp.append(String.format("		dep.market_ptr=(select id from markets where code=?) "));
        sp.append(String.format("		and "));
		sp.append(String.format("		dep.day = ? "));
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		DayEndPrice bean = null;
		try {
			conn = getConnection();
			stmt = prepareStatement(conn, sp.toString());
            {
                setString(stmt, 1, marketCode.toString());
                setDate(stmt, 2, day.getTime());
            }
			rs = executeQuery(stmt);				
			bean = getBean(rs);			
		}
		finally {
			close(conn, stmt, rs);
		}	
		return bean;
	}
		
	
	
	private DayEndPrice getBean(ResultSet rs) 
	{
        if (log.isInfoEnabled()) {
            log.info(String.format("ENTER getBean()"));
        }
		DayEndPrice bean = null;
				
		try {
			if (!rs.next()) 
            {
                if (log.isInfoEnabled()) {
                    log.info(String.format("ResultSet is empty!"));
                }
            }
            else
			{	
                if (log.isInfoEnabled()) {
                    log.info(String.format("ResultSet has record so building bean"));
                }
				String id = rs.getString("id");
                if (log.isInfoEnabled()) {
                    log.info(String.format("id=\"%s\"", id));
                }
                
                
				Calendar createdOn = Calendar.getInstance();
				createdOn.setTimeInMillis(rs.getTimestamp("created_on").getTime());
                if (log.isInfoEnabled()) {
                    log.info(String.format("createdOn=\"%s\"", createdOn.toString()));
                }

				Calendar updatedOn = Calendar.getInstance();
				updatedOn.setTimeInMillis(rs.getTimestamp("updated_on").getTime());
                if (log.isInfoEnabled()) {
                    log.info(String.format("updatedOn=\"%s\"", updatedOn.toString()));
                }

				String marketId = rs.getString("market_id");
                if (log.isInfoEnabled()) {
                    log.info(String.format("marketId=\"%s\"", marketId));
                }

				Calendar day = Calendar.getInstance();
				day.setTimeInMillis(rs.getTimestamp("day").getTime());
                if (log.isInfoEnabled()) {
                    log.info(String.format("day=\"%s\"", day.toString()));
                }

				BigDecimal price = rs.getBigDecimal("price");
                if (log.isInfoEnabled()) {
                    log.info(String.format("price=\"%f\"", price.doubleValue()));
                }

				bean 				
					= new DayEndPrice(
						id, createdOn, updatedOn, marketId, day, price
					);
                
                if (log.isInfoEnabled()) {
                    log.info(String.format("Returning =\"%s\" with id=\"%s\"",bean.getClass().getSimpleName(),bean.getId()));
                }
			}
		} catch (Throwable t) {
			throw new RuntimeException("Problem getting DayEndPrice bean", t);
		}

		return bean;
	}
	
	
    @Override
	public void setDayEndPriceForMarket(double price, Calendar day, Market.Code marketCode) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER setDayEndPriceForMarket(%f,%s,%s)",price,day,marketCode));
		}
		
        Connection conn = null;
        PreparedStatement stmt = null;
        
        if (log.isInfoEnabled()) {
            log.info(String.format("Getting DB Connection"));
        }
        try {
            conn = getConnection();
            conn.setAutoCommit(false);
        } catch (Throwable t) {           
            close(conn, stmt, null); 
            throw new RuntimeException("Unable to get a DB connection",t);
        }         
        
        if (log.isInfoEnabled()) {
            log.info(String.format("Building DELETE String"));
        }
        StringBuilder sp = new StringBuilder();
        sp.append(String.format(" delete from day_end_prices "));
        sp.append(String.format(" where "));
        sp.append(String.format("   market_ptr=(select id from markets where code=?) "));
        sp.append(String.format(" and "));
        sp.append(String.format("   day=? "));
        
        if (log.isInfoEnabled()) {
            log.info(String.format("Attempting to DELETE..."));
        }
        try {
            stmt = prepareStatement(conn, sp.toString());
            // market id
            if (log.isInfoEnabled()) {
                log.info(String.format("market_ptr=\"%s\"",marketCode.toString()));
            }
            setString(stmt, 1, marketCode.toString());
            
            // day
            if (log.isInfoEnabled()) {
                log.info(String.format("day=\"%s\"",day.getTime().toString()));
            }
            setDate(stmt, 2, day.getTime());            
            
            int updated = executeUpdate(stmt);                        
            if (log.isInfoEnabled()) {
                log.info(String.format("Deleted %d rows",updated));
            }            
            
            stmt.close();
            stmt = null;
        } 
        catch (Throwable t) {
            close(conn, stmt, null); 
            throw new RuntimeException("Problem deleting from day_end_prices ",t);
        }
        
        if (log.isInfoEnabled()) {
            log.info(String.format("Building INSERT String"));
        }
        sp.setLength(0);
        sp.append(String.format(" insert into day_end_prices "));
        sp.append(String.format(" (id, created_on, updated_on, market_ptr, day, price) "));
        sp.append(String.format(" values "));
        sp.append(String.format(" (?, ?, ?, (select id from markets where code=?), ?, ?) "));
        
        if (log.isInfoEnabled()) {
            log.info(String.format("Attempting to INSERT..."));
        }
        try {
            stmt = prepareStatement(conn, sp.toString());
            int idx = 1;
            Calendar now = getCalendarHandler().getNow();

            // id
            String id = getPrimaryKeyHandler().getPrimaryKey();
            if (log.isInfoEnabled()) {
                log.info(String.format("id=\"%s\"",id));
            }
            setString(stmt, idx++, id);
            // created on
            if (log.isInfoEnabled()) {
                log.info(String.format("created_on=\"%s\"",now.toString()));
            }
            setTimestamp(stmt, idx++, now);
            // updated on
            if (log.isInfoEnabled()) {
                log.info(String.format("updated_on=\"%s\"",now.toString()));
            }
            setTimestamp(stmt, idx++, now);
            // market code
            if (log.isInfoEnabled()) {
                log.info(String.format("'market code'=\"%s\"",marketCode.toString()));
            }
            setString(stmt, idx++, marketCode.toString());
            // day
            if (log.isInfoEnabled()) {
                log.info(String.format("day=\"%s\"",day.toString()));
            }
            setDate(stmt, idx++, day.getTime());
            // price
            if (log.isInfoEnabled()) {
                log.info(String.format("price=\"%f\"",price));
            }
            setDouble(stmt, idx++, price);

            int updated = executeUpdate(stmt);			
            if (log.isInfoEnabled()) {
                log.info(String.format("Inserted %d rows",updated));
            }     
            
            conn.commit();
        }
        catch (Throwable duck) {
            try {
                conn.rollback();
            } catch (Throwable frog) {
                log.warn("Unable to rollback transaction", frog);
            }
            throw new RuntimeException(duck.getMessage(), duck);
        }
        finally {
            close(conn, stmt, null);
        }	

	}
}
