/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Calendar;
import java.util.Date;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.FuturesExpirations;
import org.ferris.yahoo.finance.jmce.bean.Market;

/**
 *
 * @author Michael Remijan [mjremijan@yahoo.com]
 */
public class FuturesExpirationsHandlerBySql extends AbstractHandlerBySql implements FuturesExpirationsHandler
{
	private static final Logger log = Logger.getLogger(FuturesExpirationsHandlerBySql.class);
    
    private CalendarHandler calendarHandler;
       
	
	@Override
	public FuturesExpirations getForCrudeOil() 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getForCrudeOil()"));
		}	
		
		Date now = getNow();
		Market.Code tickerSymbol = Market.Code.cl;
		return getFuturesExpirations(now, tickerSymbol);
	}
	
	
	public Date getNow() {
		return getCalendarHandler().getNow().getTime();
	}
	
		
	private FuturesExpirations getFuturesExpirations(Date now, Market.Code tickerSymbol) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getFuturesExpirations(%s,%s)",String.valueOf(now), tickerSymbol));
		}
		
		StringBuilder sp = new StringBuilder();
		sp.append(String.format(" select "));
		sp.append(String.format("		fe.id, fe.created_on, fe.updated_on, m.code as ticker_symbol, contract_description, contract_month, contract_year, last_trading_day "));
		sp.append(String.format(" from "));
		sp.append(String.format("   futures_expirations fe "));
        sp.append(String.format("   , markets           m "));        
		sp.append(String.format(" where "));
        sp.append(String.format("       fe.market_ptr=m.id "));
        sp.append(String.format("		and "));
		sp.append(String.format("		fe.market_ptr = (select id from markets where code = ?) "));
		sp.append(String.format("		and "));
		sp.append(String.format("		fe.last_trading_day >= ? "));
        sp.append(String.format(" order by " ));
        sp.append(String.format("		fe.last_trading_day asc "));
		sp.append(String.format(" limit 1 "));
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		FuturesExpirations bean = null;
		try {
			conn = getConnection();
			stmt = prepareStatement(conn, sp.toString());
			
			setString(stmt,1,tickerSymbol.toString());
			setDate(stmt,2,now);
			
			rs = executeQuery(stmt);				
			bean = getBean(rs);			
		}
		finally {
			close(conn, stmt, rs);
		}	
		return bean;
	}
		
	
	
	private FuturesExpirations getBean(ResultSet rs) 
	{
        if (log.isInfoEnabled()) {
            log.info(String.format("Enter getBean(..)"));
        }
		FuturesExpirations bean = null;
				
		try {
			if (!rs.next()) 
            {
                if (log.isInfoEnabled()) {
                    log.info(String.format("ResultSet is empty!"));
                }
            }
            else
			{	
                if (log.isInfoEnabled()) {
                    log.info(String.format("ResultSet has records, building bean"));
                }
				String id 
					= rs.getString("id");

				Calendar createdOn = Calendar.getInstance();
				createdOn.setTimeInMillis(rs.getTimestamp("created_on").getTime());

				Calendar updatedOn = Calendar.getInstance();
				createdOn.setTimeInMillis(rs.getTimestamp("updated_on").getTime());

				String tickerSymbol
					= rs.getString("ticker_symbol");
				
				String contractDescription
					= rs.getString("contract_description");
				
				char contractMonth
					= rs.getString("contract_month").charAt(0);

				int contractYear 
					= rs.getInt("contract_year");

				Date lastTradingDay
					= rs.getDate("last_trading_day");

				bean 				
					= new FuturesExpirations(
						id, createdOn, updatedOn, tickerSymbol, contractDescription, contractMonth, contractYear, lastTradingDay
					);	
                if (log.isInfoEnabled()) {
                    log.info(String.format("Built bean=\"%s\" with id=\"%s\"",bean.getClass().getSimpleName(),bean.getId()));
                }
			}
		} catch (Throwable t) {
			throw new RuntimeException("Problem getting FuturesExpirations bean", t);
		}

		return bean;
	}

    /**
     * @return the calendarHandler
     */
    public CalendarHandler getCalendarHandler() {
        return calendarHandler;
    }

    /**
     * @param calendarHandler the calendarHandler to set
     */
    public void setCalendarHandler(CalendarHandler calendarHandler) {
        this.calendarHandler = calendarHandler;
    }
	
}
