/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.handler;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Calendar;
import java.util.Date;
import org.apache.log4j.Logger;
import org.ferris.yahoo.finance.jmce.bean.Market;
import org.ferris.yahoo.finance.jmce.bean.YearEndPrice;

/**
 *
 * @author Michael Remijan [mjremijan@yahoo.com]
 */
public class YearEndPriceHandlerBySql extends AbstractHandlerBySql implements YearEndPriceHandler
{
	private static final Logger log = Logger.getLogger(YearEndPriceHandlerBySql.class);
	
	public CalendarHandler calendarHandler;
	public CalendarHandler getCalendarHandler() {
		return calendarHandler;
	}
	public void setCalendarHandler(CalendarHandler calendarHandler) {
		this.calendarHandler = calendarHandler;
	}
    
    private PrimaryKeyHandler primaryKeyHandler;    
    public PrimaryKeyHandler getPrimaryKeyHandler() {
        return primaryKeyHandler;
    }
    public void setPrimaryKeyHandler(PrimaryKeyHandler primaryKeyHandler) {
        this.primaryKeyHandler = primaryKeyHandler;
    } 

	
	@Override
	public YearEndPrice getYearEndPriceForGold(int year) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getYearEndPriceForGold(%d)",year));
		}		
		
		return getYearEndPriceForMarket(year, Market.Code.gc);
	}
	
	@Override
	public YearEndPrice getYearEndPriceForOil(int year) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getYearEndPriceForOil(%d)",year));
		}		
		
		return getYearEndPriceForMarket(year, Market.Code.cl);
	}
	
	@Override
	public YearEndPrice getYearEndPriceForStandardAndPoors500(int year) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getYearEndPriceForStandardAndPoors500(%d)",year));
		}		
		
		return getYearEndPriceForMarket(year, Market.Code.sp);
	}
	
	@Override
	public YearEndPrice getYearEndPriceForNasdaq(int year) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getYearEndPriceForNasdaq(%d)",year));
		}		
		
		return getYearEndPriceForMarket(year, Market.Code.nas);
	}
	
	
	@Override
	public YearEndPrice getYearEndPriceForDow(int year) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getYearEndPriceForDow(%d)",year));
		}
		
		return getYearEndPriceForMarket(year, Market.Code.dow);
	}
	
		
	public YearEndPrice getYearEndPriceForMarket(int year, Market.Code marketCode) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER getYearEndPriceForMarket(%d,%s)",year,marketCode));
		}
		
		StringBuilder sp = new StringBuilder();
		sp.append(String.format(" select "));
		sp.append(String.format("		yep.id, yep.created_on, yep.updated_on, m.id as market_id, yep.year, yep.price "));
		sp.append(String.format(" from "));
		sp.append(String.format("   year_end_prices yep "));
        sp.append(String.format(" , markets m "));
		sp.append(String.format(" where "));                
        sp.append(String.format("		yep.market_ptr=m.id "));        		        
		sp.append(String.format("		and "));
        sp.append(String.format("		yep.market_ptr=(select id from markets where code=?) "));
        sp.append(String.format("		and "));
		sp.append(String.format("		yep.year = ? "));
		
		Connection conn = null;
		PreparedStatement stmt = null;
		ResultSet rs = null;
		YearEndPrice bean = null;
		try {
			conn = getConnection();
			stmt = prepareStatement(conn, sp.toString());
            {
                setString(stmt, 1, marketCode.toString());
                setInt(stmt,2,year);
            }
			rs = executeQuery(stmt);				
			bean = getBean(rs);			
		}
		finally {
			close(conn, stmt, rs);
		}	
		return bean;
	}
		
	
	
	private YearEndPrice getBean(ResultSet rs) 
	{
        if (log.isInfoEnabled()) {
            log.info(String.format("ENTER getBean()"));
        }
		YearEndPrice bean = null;
				
		try {
			if (!rs.next()) 
            {
                if (log.isInfoEnabled()) {
                    log.info(String.format("ResultSet is empty!"));
                }
            }
            else
			{	
                if (log.isInfoEnabled()) {
                    log.info(String.format("ResultSet has record so building bean"));
                }
				String id 
					= rs.getString("id");

				Calendar createdOn = Calendar.getInstance();
				createdOn.setTimeInMillis(rs.getTimestamp("created_on").getTime());

				Calendar updatedOn = Calendar.getInstance();
				createdOn.setTimeInMillis(rs.getTimestamp("updated_on").getTime());

				String marketId 
					= rs.getString("market_id");

				int year 
					= rs.getInt("year");

				BigDecimal price
					= rs.getBigDecimal("price");

				bean 				
					= new YearEndPrice(
						id, createdOn, updatedOn, marketId, year, price
					);
                
                if (log.isInfoEnabled()) {
                    log.info(String.format("Returning Bean=\"%s\" with id=\"%s\"",bean.getClass().getSimpleName(),bean.getId()));
                }
			}
		} catch (Throwable t) {
			throw new RuntimeException("Problem getting YearEndPrice bean", t);
		}

		return bean;
	}
	
	
    @Override
	public void setYearEndPriceForMarket(double price, int year, Market.Code marketCode) 
	{
		if (log.isInfoEnabled()) {
			log.info(String.format("ENTER setYearEndPriceForMarket(%f, %d,%s)",price,year,marketCode));
		}
		
		boolean lastDayOfYear
			= getCalendarHandler().isNowTheLastDayOfTheYear();
		
		if (!lastDayOfYear) {
			log.info(String.format("Today is not the last day of the year...NOT saving (%f, %d,%s)",price,year,marketCode));			
		} 
		else 
		{
			log.info(String.format("Yey...last day of the year!!!"));			
			
			StringBuilder sp = new StringBuilder();
			sp.append(String.format(" insert into year_end_prices "));
			sp.append(String.format(" (id, created_on, updated_on, market_ptr, year, price) "));
			sp.append(String.format(" values "));
			sp.append(String.format(" (?, ?, ?, (select id from markets where code=?), ?, ?) "));

			Connection conn = null;
			PreparedStatement stmt = null;
			try {
				conn = getConnection();
				stmt = prepareStatement(conn, sp.toString());
				int idx = 1;
				Calendar now = getCalendarHandler().getNow();

				// id
				setString(stmt, idx++, getPrimaryKeyHandler().getPrimaryKey());
				// created on
				setTimestamp(stmt, idx++, now);
				// updated on
				setTimestamp(stmt, idx++, now);
				// market id
				setString(stmt, idx++, marketCode.toString());
				// year
				setInt(stmt, idx++, year);
				// price
				setDouble(stmt, idx++, price);

				int updated = executeUpdate(stmt);			
                if (log.isInfoEnabled()) {
                    log.info(String.format("ExcuteUpdate updated %d rows",updated));
                }
			}
			finally {
				close(conn, stmt, null);
			}	
		}
	}



	

}
