/*
 * Copyright 2011 The Ferris Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ferris.yahoo.finance.jmce.net;

import java.io.InputStream;
import java.net.CookieHandler;
import java.net.CookieManager;
import java.net.URL;
import java.util.Arrays;
import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.ferris.lang.string.StringTools;

/**
 *
 * @author mjr2586
 */
public class HtmlPageContents 
{
    private static final Logger log = Logger.getLogger(HtmlPageContents.class);
    
    private String pageContentsAsHtml;
    
    public HtmlPageContents(){}
    
    public HtmlPageContents(String urlAsString) 
    {
        InputStream is = null;
        try {
            if (log.isInfoEnabled()) {
                log.info(String.format("ENTER HtmlPageContents<constructor>(%s)",urlAsString));
            }
            
            log.info(String.format("Set Default CookieManager"));
            CookieManager cm = new CookieManager();
            CookieHandler.setDefault(cm);

            if (log.isInfoEnabled()) {
                log.info(String.format("New URL to \"%s\"", urlAsString));
            }
            URL url = new URL(urlAsString);


            if(log.isInfoEnabled()) {
                log.info("url.openStream()");
            }
            is = url.openStream();

            if (log.isInfoEnabled()) {
                log.info("Convert InputStream to String");
            }        
            pageContentsAsHtml = IOUtils.toString(is);

            if (log.isInfoEnabled()) {
                log.info(String.format("\n----- BEGIN: PAGE CONTENTS -----\n%s\n----- END: PAGE CONTENTS -----\n", pageContentsAsHtml));
            }
            
        } 
        catch(Throwable t) {
            log.fatal("Failed constructing new HtmlPageContents object", t);
            throw new RuntimeException(t);
        } 
        finally {
            IOUtils.closeQuietly(is);
        }
    }

    /**
     * @return the pageContentsAsHtml
     */
    public String getPageContentsAsHtml() {
        return pageContentsAsHtml;
    }
    
    
    /**
     * @param pageContentsAsHtml the pageContentsAsHtml to set
     */
    public void setPageContentsAsHtml(String pageContentsAsHtml) {
        this.pageContentsAsHtml = pageContentsAsHtml;
    }    
    
    
    /**
     * Look in data for the span with the given ID and return a substring
     * of data which is just that span.
     * 
     * @param id
     * @param data
     * @return 
     */
    private String getSnippet(String[] ids)
    {
        //<span id="yfs_c10_^dji" class="c10">
        //  <span class="yfi-price-change-red">-74.70</span>
        //</span>        
        int indexOfId = 0;
        for (int i=0; i<ids.length; i++)
        {
            indexOfId = getPageContentsAsHtml().indexOf(ids[i],indexOfId);
            log.info(String.format("Index of \"%s\" is %d",ids[i], indexOfId));
        }
        
        if (indexOfId < 0) {
            if (log.isInfoEnabled()) {
                log.info(String.format("IDs \"%s\" not found!", Arrays.asList(ids)));
            }
            return "";
        }
        
        // ** NOTE **
        // This code does NOT assume the page has a <span> tag!  This is because
        // in one instance the page uses a <b> tag instead of a <span>.  So, 
        // instead of specfically looking for a <span>, the code will look for
        // the < which is the opening of whatever tag it may be.
        int startIndexOfOpenTag = StringTools.indexOfBefore(getPageContentsAsHtml(), "<", indexOfId);
        log.info(String.format("Start index of open tag \"<\" %d", startIndexOfOpenTag));
        
        // ** NOTE **
        // This code does NOT assume the page has a closing </span> tag!  This is
        // because in one instance the page uses a </b> instead of a </span>. So,
        // I look for the start of the closing tag by looking for </ I then find
        // the end of the tag by searching for > and use substring to get the 
        // acutal name of the closing tag.
        int startIndexOfClosingTag = getPageContentsAsHtml().indexOf("</", indexOfId);
        log.info(String.format("Start index of closing tag \"</\" %d", startIndexOfClosingTag));
        int endIndexOfClosingTag = getPageContentsAsHtml().indexOf(">", startIndexOfClosingTag);
        log.info(String.format("End index of closing tag \"%s\" is %d",  getPageContentsAsHtml().substring(startIndexOfClosingTag, endIndexOfClosingTag + 1), endIndexOfClosingTag));
        
        String span = getPageContentsAsHtml().substring(startIndexOfOpenTag, endIndexOfClosingTag + 1);
        log.info(String.format("Returning span=\"%s\"",span));
        
        return span;       
    }
    
    private String removeImg(String lineOfData)
    {
        // look for and remove an <img tag if it exists
        int a = lineOfData.indexOf("<img");
        if (a >= 0) 
        {
            log.info(String.format("Index of <img is %d", a));
            int b = lineOfData.indexOf(">", a);
            log.info(String.format("Index of > is %d", b));
            StringBuilder sp = new StringBuilder(lineOfData);
            sp = sp.replace(a, b + 1, "");
            lineOfData = sp.toString();
            log.info(String.format("Now lineOfData=\"%s\"", lineOfData));
        }
        return lineOfData;
    }
    
    public String getContentsOfTagByIds(String...ids)                 
    {
        String lineOfData = getSnippet(ids);
        
        // ** NOTE **
        // This code is actually tag ignostic.  It can be a
        // <span>, <b>, <div>, whatever.  The code will simply
        // get whatever is between the > and < charactes.
        
        lineOfData = removeImg(lineOfData);
        
        int b = lineOfData.indexOf('>',0);
        log.info(String.format("Index of > is %d", b));
        
		int e = lineOfData.indexOf('<', b);
        log.info(String.format("Index of < is %d", e));
        
		String contents = lineOfData.substring(b + 1,e).trim();
        if (contents.startsWith("(")) {
            contents = contents.substring(1);
        }
        if (contents.endsWith(")")) {
            contents = contents.substring(0,contents.length() - 1);
        }
               
        log.info(String.format("Returning contents=\"%s\"",contents));
        
        return contents;		
	}


    public boolean contains(String whatToLookFor)
    {
        int idx = getPageContentsAsHtml().indexOf(whatToLookFor);
        return (idx >= 0);
    }
    
}
